"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRESTDataSource = void 0;
const datasource_rest_1 = require("@apollo/datasource-rest");
const utils_keyvaluecache_1 = require("@apollo/utils.keyvaluecache");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const correlation_id_1 = __importDefault(require("correlation-id"));
const misc_1 = require("../misc");
const defaultCacheSetting = {
    max: 500,
    // for use with tracking overall storage size
    maxSize: 5000,
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    sizeCalculation: (value, key) => {
        return 1;
    },
    // 5 seconds (in milliseconds)
    ttl: 5 * 1000
};
class BaseRESTDataSource extends datasource_rest_1.RESTDataSource {
    token;
    cache;
    constructor(options) {
        super(options);
        this.token = options?.token;
        this.cache = options?.cache || new utils_keyvaluecache_1.InMemoryLRUCache(defaultCacheSetting);
    }
    willSendRequest(request) {
        if (!request.headers['authorization'])
            request.headers['authorization'] = this.token;
        request.headers['x-correlation-id'] = correlation_id_1.default.getId();
    }
    async invokeGraphqlWithCache(functionName, body, accessInfo, allowCache = false, ttlSeconds = 5) {
        if (allowCache) {
            const key = JSON.stringify({ functionName, query: body });
            const value = await this.cache.get(key);
            if (value) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'using cached value');
                return value;
            }
            else {
                const data = await this.invokeGraphql(functionName, body, accessInfo);
                await this.cache.set(key, data, { ttl: ttlSeconds });
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'cache set!');
                return data;
            }
        }
        else {
            return this.invokeGraphql(functionName, body, accessInfo);
        }
    }
    async invokeGraphql(functionName, body, accessInfo) {
        let data = null;
        try {
            const { data: newData } = await this.post('/graphql', {
                headers: {
                    authorization: accessInfo?.token
                },
                body
            });
            data = newData[functionName];
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeRest error:${JSON.stringify(error, null, 2)}`);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `This is what i get from ${functionName}`);
        return data;
    }
    async invokeRestWithCache(functionName, path, body, accessInfo, allowCache = false, ttlSeconds = 5) {
        if (allowCache) {
            const key = JSON.stringify({ functionName, path, body });
            const value = await this.cache.get(key);
            if (value) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'using cached value');
                return value;
            }
            else {
                const data = await this.invokeRest(functionName, path, body, accessInfo);
                await this.cache.set(key, data, { ttl: ttlSeconds });
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'cache set!');
                return data;
            }
        }
        else {
            return this.invokeRest(functionName, path, body, accessInfo);
        }
    }
    async invokeRest(functionName, path, body, accessInfo) {
        let data = null;
        try {
            const { data: newData } = await this.post(`/${path}`, {
                headers: {
                    authorization: accessInfo?.token
                },
                body
            });
            data = newData;
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeRest error:${JSON.stringify(error, null, 2)}`);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `This is what i get from ${functionName}`);
        return data;
    }
}
exports.BaseRESTDataSource = BaseRESTDataSource;
//# sourceMappingURL=base.data-source.js.map